@@ -68,12 +68,17 @@ void *alloca (size_t);
 #include <Ecore.h>
 #include <Ecore_Evas.h>
 #include <Ecore_File.h>
+#include <Ecore_Input.h>
 #ifdef HAVE_ECORE_IMF
 # include <Ecore_IMF.h>
 # include <Ecore_IMF_Evas.h>
 #endif
 #include <Embryo.h>
 
+#ifdef HAVE_EIO
+# include <Eio.h>
+#endif
+
 #include "Edje.h"
 
 EAPI extern int _edje_default_log_dom ;
@@ -191,7 +196,7 @@ struct _Edje_Smart_Api
 /* increment this when you add new feature to edje file format without
  * breaking backward compatibility.
  */
-#define EDJE_FILE_MINOR 3
+#define EDJE_FILE_MINOR 4
 
 /* FIXME:
  *
@@ -317,6 +322,7 @@ typedef struct _Edje_Part_Description_Spec_Box       Edje_Part_Description_Spec_
 typedef struct _Edje_Part_Description_Spec_Table     Edje_Part_Description_Spec_Table;
 typedef struct _Edje_Patterns                        Edje_Patterns;
 typedef struct _Edje_Part_Box_Animation              Edje_Part_Box_Animation;
+typedef struct _Edje_Part_Limit                      Edje_Part_Limit;
 
 typedef struct _Edje Edje;
 typedef struct _Edje_Real_Part_State Edje_Real_Part_State;
@@ -448,7 +454,9 @@ struct _Edje_File
    Edje_Image_Directory           *image_dir;
    Edje_Sound_Directory           *sound_dir;
    Eina_List                      *styles;
+
    Eina_List                      *color_classes;
+   Eina_Hash                      *color_hash;
 
    int                             references;
    const char                     *compiler;
@@ -466,6 +474,13 @@ struct _Edje_File
 
    Eet_File                       *ef;
 
+#ifdef HAVE_EIO
+   Eio_Monitor                    *monitor;
+   Eina_List                      *edjes;
+   Eina_List                      *handlers;
+   Ecore_Timer                    *timeout;
+#endif
+
    unsigned char                   free_strings : 1;
    unsigned char                   dangling : 1;
    unsigned char		   warning : 1;
@@ -664,6 +679,7 @@ struct _Edje_Limit
       TYPE      GROUP;            \
       TYPE      BOX;              \
       TYPE      TABLE;            \
+      TYPE      SPACER;	  \
       TYPE      EXTERNAL;
 
 struct _Edje_Part_Collection_Directory_Entry
@@ -714,6 +730,21 @@ struct _Edje_Pack_Element
    unsigned short   colspan, rowspan;
 };
 
+typedef enum {
+  EDJE_PART_LIMIT_UNKNOWN = 0,
+  EDJE_PART_LIMIT_BELOW,
+  EDJE_PART_LIMIT_ZERO,
+  EDJE_PART_LIMIT_OVER
+} Edje_Part_Limit_State;
+
+struct _Edje_Part_Limit
+{
+   int part;
+
+   Edje_Part_Limit_State width; /* -1, 0 or 1 */
+   Edje_Part_Limit_State height; /* -1, 0, or 1 */
+};
+
 /*----------*/
 
 struct _Edje_Part_Collection
@@ -741,6 +772,9 @@ struct _Edje_Part_Collection
 
       Edje_Limit **horizontal;
       unsigned int horizontal_count;
+
+      Edje_Part_Limit *parts;
+      unsigned int parts_count;
    } limits;
 
    Edje_Part **parts; /* an array of Edje_Part */
@@ -837,6 +871,7 @@ struct _Edje_Part
    unsigned char          select_mode;
    unsigned char          cursor_mode;
    unsigned char          multiline;
+   unsigned char          access; /* it will be used accessibility feature */
    Edje_Part_Api          api;
 };
 
@@ -901,6 +936,7 @@ struct _Edje_Part_Description_Common
    } persp;
 
    unsigned char     visible; /* is it shown */
+   unsigned char     limit; /* 0 == no, 1 = width, 2 = height, 3 = both */
 };
 
 struct _Edje_Part_Description_Spec_Fill
@@ -1088,7 +1124,7 @@ struct _Edje
    Eina_List            *actions; /* currently running actions */
    Eina_List            *callbacks;
    Eina_List            *pending_actions;
-   Eina_List            *color_classes;
+   Eina_Hash            *color_classes;
    Eina_List            *text_classes;
    /* variable pool for Edje Embryo scripts */
    Edje_Var_Pool        *var_pool;
@@ -1104,6 +1140,8 @@ struct _Edje
    int                   table_programs_size;
    unsigned int          table_parts_size;
 
+   Eina_List            *groups;
+
    struct {
       Eina_Hash         *text_class;
       Eina_Hash         *color_class;
@@ -1148,9 +1186,12 @@ struct _Edje
       void                  *data;
    } item_provider;
 
+   Eina_List            *user_defined;
+
+   int                   walking_callbacks;
+
    unsigned int          dirty : 1;
    unsigned int          recalc : 1;
-   unsigned int          walking_callbacks : 1;
    unsigned int          delete_callbacks : 1;
    unsigned int          just_added_callbacks : 1;
    unsigned int          have_objects : 1;
@@ -1191,6 +1232,7 @@ struct _Edje_Calc_Params
 	 union {
 	    struct {
 	       int           l, r, t, b; // 16
+               FLOAT_T       border_scale_by;
 	    } image; // 16
 	 } spec; // 16
       } common; // 40
@@ -1535,6 +1577,52 @@ struct _Edje_Patterns
    size_t          finals[];
 };
 
+typedef enum _Edje_User_Defined_Type 
+{
+   EDJE_USER_SWALLOW,
+   EDJE_USER_BOX_PACK,
+   EDJE_USER_TABLE_PACK,
+   EDJE_USER_STRING,
+   EDJE_USER_DRAG_STEP,
+   EDJE_USER_DRAG_PAGE,
+   EDJE_USER_DRAG_VALUE,
+   EDJE_USER_DRAG_SIZE
+} Edje_User_Defined_Type;
+
+typedef struct _Edje_User_Defined Edje_User_Defined;
+struct _Edje_User_Defined
+{
+   Edje_User_Defined_Type type;
+   const char *part;
+   Edje *ed;
+
+   union {
+      struct {
+         const char *text;
+      } string;
+      struct {
+         Evas_Object *child;
+      } swallow;
+      struct {
+         Evas_Object *child;
+         int index;
+      } box;
+      struct {
+         Evas_Object *child;
+         unsigned short col;
+         unsigned short row;
+         unsigned short colspan;
+         unsigned short rowspan;
+      } table;
+      struct {
+         double x, y;
+      } drag_position;
+      struct {
+         double w, h;
+      } drag_size;
+   } u;
+};
+
 Edje_Patterns   *edje_match_collection_dir_init(const Eina_List *lst);
 Edje_Patterns   *edje_match_programs_signal_init(Edje_Program * const *array,
 						 unsigned int count);
@@ -1607,6 +1695,7 @@ extern Eina_Mempool *_emp_GROUP;
 extern Eina_Mempool *_emp_BOX;
 extern Eina_Mempool *_emp_TABLE;
 extern Eina_Mempool *_emp_EXTERNAL;
+extern Eina_Mempool *_emp_SPACER;
 extern Eina_Mempool *_emp_part;
 
 void  _edje_part_pos_set(Edje *ed, Edje_Real_Part *ep, int mode, FLOAT_T pos, FLOAT_T v1, FLOAT_T v2);
@@ -1807,7 +1896,7 @@ void _edje_textblock_styles_del(Edje *ed);
 void _edje_textblock_style_all_update(Edje *ed);
 void _edje_textblock_style_parse_and_fix(Edje_File *edf);
 void _edje_textblock_style_cleanup(Edje_File *edf);
-Edje_File *_edje_cache_file_coll_open(const char *file, const char *coll, int *error_ret, Edje_Part_Collection **edc_ret);
+Edje_File *_edje_cache_file_coll_open(const char *file, const char *coll, int *error_ret, Edje_Part_Collection **edc_ret, Edje *ed);
 void _edje_cache_coll_clean(Edje_File *edf);
 void _edje_cache_coll_flush(Edje_File *edf);
 void _edje_cache_coll_unref(Edje_File *edf, Edje_Part_Collection *edc);
@@ -1816,6 +1905,7 @@ void _edje_cache_file_unref(Edje_File *edf);
 void _edje_embryo_globals_init(Edje *ed);
 
 #define CHKPARAM(n) if (params[0] != (sizeof(Embryo_Cell) * (n))) return -1;
+#define HASNPARAMS(n) (params[0] == (sizeof(Embryo_Cell) * (n)))
 #define GETSTR(str, par) { \
    Embryo_Cell *___cptr; \
    int ___l; \
@@ -2053,6 +2143,7 @@ void _edje_lib_ref(void);
 void _edje_lib_unref(void);
 
 void _edje_subobj_register(Edje *ed, Evas_Object *ob);
+void _edje_subobj_unregister(Edje *ed, Evas_Object *ob);
 
 void _edje_multisense_init(void);
 void _edje_multisense_shutdown(void);
@@ -2061,6 +2152,9 @@ Eina_Bool _edje_multisense_internal_sound_tone_play(Edje *ed, const char *tone_n
 
 void _edje_part_recalc(Edje *ed, Edje_Real_Part *ep, int flags, Edje_Calc_Params *state);
 
+void _edje_user_definition_remove(Edje_User_Defined *eud, Evas_Object *child);
+void _edje_user_definition_free(Edje_User_Defined *eud);
+
 #ifdef HAVE_LIBREMIX
 #include <remix/remix.h>
 #endif
